package be.bitscuit.gpslogger;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.os.Binder;
import android.os.IBinder;
import android.os.Looper;

import androidx.annotation.Nullable;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationAvailability;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;

import java.util.ArrayList;

public class LoggerService extends Service {

    //Binder vars
    private IBinder binder = new LoggerBinder();
    private LocationEventListener eventListener;

    //Location vars
    private FusedLocationProviderClient fusedLocationClient;
    private ArrayList<LocationEvent> locationEvents;

    @Override
    public void onCreate() {
        super.onCreate();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {

        //Make foreground
        Notification notification =
                new Notification.Builder(this)
                        .setContentTitle("Logging GPS location data!")
                        .setContentText("GPS Logger is running...")
                        .setSmallIcon(R.drawable.location_on)
                        .build();

        startForeground(100, notification);

        //Init vars
        locationEvents = new ArrayList<>();

        //Request location updates
        fusedLocationClient = LocationServices.getFusedLocationProviderClient(this);

        LocationRequest locationRequest = LocationRequest.create();
        locationRequest.setInterval(3000);
        locationRequest.setFastestInterval(1000);
        locationRequest.setSmallestDisplacement(1);//1 meter displacement between 2 locations
        locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);

        try {
            fusedLocationClient.requestLocationUpdates(locationRequest, locationCallback, Looper.getMainLooper());
        }catch (SecurityException e){}


        //Make sticky
        return START_STICKY;
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return binder;
    }

    @Override
    public boolean onUnbind(Intent intent) {

        //Reset eventListener
        eventListener = null;

        return super.onUnbind(intent);
    }

    @Override
    public void onDestroy() {

        //Remove location updates
        fusedLocationClient.removeLocationUpdates(locationCallback);

        super.onDestroy();
    }




    private LocationCallback locationCallback = new LocationCallback(){
        @Override
        public void onLocationResult(LocationResult locationResult) {
            //When got location update
            if(locationResult == null) return;

            //Analyse locations
            for(Location location : locationResult.getLocations()) {
                LocationEvent locationEvent = new LocationEvent(
                        location.getLatitude(),
                        location.getLongitude(),
                        location.getAltitude(),
                        location.getSpeed(),
                        location.getTime()
                );

                //Add LocationEvent to array
                locationEvents.add(locationEvent);

                //Call listener if set
                try {
                    eventListener.onLocationEvent(locationEvent);
                }catch (Exception e){}
            }
        }
    };



    //Methods Binder can access to communicate with this Service
    public void setLocationEventListener(LocationEventListener eventListener){
        this.eventListener = eventListener;
    }

    public ArrayList<LocationEvent> getLocationEvents(){
        return locationEvents;
    }



    //Binder class
    public class LoggerBinder extends Binder {
        public LoggerService getService(){
            return LoggerService.this;
        }
    }

}
